import argparse
import sys

from zeversolarlocal import cli  # noqa


def _parse_args(args):
    parser = argparse.ArgumentParser()
    parser.add_argument("ipaddress")

    args = parser.parse_args(args)
    return args


if __name__ == "__main__":
    args = _parse_args(sys.argv[1:])
    cli.get(args)



import pytest

from zeversolarlocal.api import (
    SolarData,
    ZeverError,
    _convert_to_string,
    _parse_content,
    default_url,
    httpx_get_client,
    solardata,
)


def test_convert_to_string():
    result = _convert_to_string(b"abc")

    assert result == "abc"


def test_parse_incoming_success():
    incoming = "1 1 EAB9618C1399 AWWQBBWVVXDJWVXF M11 18625-797R+17829-719R 12:41 24/08/2021 1 1 ZS150060118C0109 1185 3.14 OK Error"

    result = _parse_content(incoming)

    assert result == SolarData(3.14, 1185)


@pytest.mark.parametrize(
    "incoming",
    [
        "1 1 EAB9618C1399 AWWQBBWVVXDJWVXF M11 18625-797R+17829-719R 12:41 24/08/2021 1 1 ZS150060118C0109 1185",  # too short
        "1 1 EAB9618C1399 AWWQBBWVVXDJWVXF M11 18625-797R+17829-719R 12:41 24/08/2021 1 1 ZS150060118C0109 1185 3.a4 OK Error",  # wrong float
        "1 1 EAB9618C1399 AWWQBBWVVXDJWVXF M11 18625-797R+17829-719R 12:41 24/08/2021 1 1 ZS150060118C0109 1a85 3.14 OK Error",  # wrong int
    ],
)
def test_parse_fail_incorrect_incoming(incoming):
    with pytest.raises(ZeverError):
        _parse_content(incoming)


@pytest.mark.asyncio
async def test_connection_timeout():
    ip_addr = "http://192.168.188.29/home.html"

    with pytest.raises(ZeverError):
        await httpx_get_client(ip_addr, 0.5)


def test_default_url():
    ip = "192.168.1.12"

    result = default_url(ip)

    assert result == "http://192.168.1.12/home.cgi"


@pytest.mark.asyncio
async def test_solardata():
    async def a_get_client(url: str, *args, **kwargs):
        return b"1 1 EAB9618C1399 AWWQBBWVVXDJWVXF M11 18625-797R+17829-719R 12:41 24/08/2021 1 1 ZS150060118C0109 1185 3.14 OK Error"

    result = await solardata("fakeurl", client=a_get_client)

    assert isinstance(result, SolarData)



from zeversolarlocal import cli
from zeversolarlocal.__main__ import _parse_args
from zeversolarlocal.api import SolarData


def test_parse_args():
    input = ["192.168.1.12"]

    result = _parse_args(input)

    assert result.ipaddress == "192.168.1.12"


def test_cli(monkeypatch):
    data = SolarData(10, 10)

    async def update(*args, **kwargs):
        return data

    monkeypatch.setattr(cli.zeversolarlocal, "solardata", update)
    args = _parse_args(["192.168.1.12"])

    result = cli.get(args)
    assert result == data



"""zeversolarlocal API"""

from dataclasses import dataclass
from typing import Optional

import httpx

DAILY_ENERGY_IDX = 12
CURRENT_POWER_INDEX = 11


@dataclass
class SolarData:
    daily_energy: float  # kiloWatthour
    current_power: int  # Watt


def _convert_to_string(data: bytes) -> str:
    return data.decode(encoding="utf-8")


def _parse_content(incoming: str) -> SolarData:
    """Parse incoming data from the inverter.

    Incoming data is a string in form of:
    1 1 EAB9618C1399 AWWQBBWVVXDJWVXF M11 18625-797R+17829-719R 12:41 24/08/2021 1 1 ZS150060118C0109 1185 3.14 OK Error

    0 1      2               3         4            5             6       7      8 9        10         11   12  13  14
    """

    data = incoming.split()
    try:
        _daily_energy = float(data[DAILY_ENERGY_IDX])
        _current_power = int(data[CURRENT_POWER_INDEX])
    except (ValueError, IndexError) as err:
        raise ZeverError(err) from None
    else:
        return SolarData(_daily_energy, _current_power)


class HttpxClient:
    def __init__(self) -> None:
        self.client: Optional[httpx.AsyncClient] = None

    async def __aenter__(self):
        self.client = httpx.AsyncClient()
        return self.client

    async def __aexit__(self, exc_type, exc, tb):
        if self.client:
            await self.client.aclose()
        if exc_type:
            raise ZeverError(exc)


async def httpx_get_client(url: str, timeout=2) -> bytes:
    try:
        async with httpx.AsyncClient() as client:
            data = await client.get(url, timeout=timeout)
    except httpx.TimeoutException:
        raise ZeverError(
            "Connection to Zeversolar inverter timed out. %s", url
        ) from None
    return data.content


def default_url(ipaddress: str):
    """Return the default url based on the provided ip address
    Address only ie. 192.168.1.3"""

    return f"http://{ipaddress}/home.cgi"


async def solardata(url: str, client=None) -> SolarData:
    """Query the local zever solar inverter for new data.

    Raises:
        A ZeverError when unable to fetch data.
    """
    if client is None:
        client = httpx_get_client

    data = await client(url)

    return _parse_content(_convert_to_string(data))


class ZeverError(Exception):
    """Parsing problem"""



"""Zever solar cli."""
import asyncio

import zeversolarlocal


def get(args):
    loop = asyncio.get_event_loop()
    address = args.ipaddress
    url = zeversolarlocal.default_url(address)

    print(f"Getting solar data from Zeversolar inverter located at {address}")
    solar_data = loop.run_until_complete(zeversolarlocal.solardata(url))
    print(solar_data)
    return solar_data
